# Example of NAO analysis in EC-Earth v3.1 simulations.
#######################################################
#
# Martin Ménégoz, June 2015 <martin.menegoz@ic3.cat>
#
# NB1: There are different ways to define the NAO. Here, we compute the NAO according to EOF projections (on observations) and Sea Level Pressure differences over two different domains (Hurrell, 2003 and Stephenson, 2006). The results are similar for the 3 NAO definitions. This is shown by the 3 plots made with this script.
# The EOF definition is based on CFU_EOF and CFU_MODE that has been designed by:
# Fabian Lienert <flienert@ic3.cat>
# Lauriane Batte <lauriane.batte@ic3.cat>
# Virginie Guemas <virginie.guemas@ic3.cat>

# NB2: Each user has to check how he want to define the NAO depending on its needs.

# Forecasts used for this example:
###################################
# m04o: low resolution; initialisation with soil moisture climatology
# m04r: low resolution, initialisation with soil moisture observation
# m04p: high resolution; initialisation with soil moisture climatology
# m04s: high resolution, initialisation with soil moisture observation

##############################
# Options for NAO calculation:
##############################

# Projection onto model EOF (1) or observation EOF (0)?
naocalc=0

# Seasonal means:
fcavt=2:4 # DJF

# SPECS domain (Stephenson et al., 2006)
lonmin <- 270; lonmax <- 60; latmin <- 20; latmax <- 90

## NAO Hurrell's domain (Hurrell 2003)
#lonmin <- 270; lonmax <- 40; latmin <- 20; latmax <- 80

################################################

# Libraries
library(s2dverification)
library(RColorBrewer)
# Using CFU library for NAO computation (routines CFU_EOF and CFU_MODE)
source('~/git-stuff/cfutools/eigentechniques/CFU_Rfunc.txt')

# Startdate in November for winter NAO forecasts:
sdates_new <- paste(seq(as.Date("1993/11/01"),as.Date("2009/11/01"), "years"),sep="-")
sdates_ok <- gsub("-","",sdates_new)
# It should be better to have more years, but here, the EOF computed over 17 startdates is similar to those computed over 30 years.

# Loading data:
var='psl'
Data_SLP=Load(var,c("m04o","m04r","m04p","m04s"),obs=c("ERAint"),sdates=sdates_ok,output="lonlat",storefreq = "monthly",nleadtime=4,leadtimemin=1,leadtimemax=4,lonmin=lonmin,lonmax=lonmax,latmin=latmin,latmax=latmax,grid='r512x256')

save.image(file = paste("Load_2d_Atlantic_",var,".RData",sep=""))

# When data loaded one time, we can do quick tests by loading this RData:
#load(file = paste("Load_2d_Atlantic_",var,".RData",sep=""))

# Clim and ano:
clims_SLP=Clim(Data_SLP$mod,Data_SLP$obs,memb='FALSE') # One common clim for all members (suitable for atmospĥeric variables)
ano_SLP_mod=Ano(Data_SLP$mod,clims_SLP$clim_exp)
ano_SLP_obs=Ano(Data_SLP$obs,clims_SLP$clim_obs)

# Lon/Lat:
lon=Data_SLP$lon
lat=Data_SLP$lat

# Dimensions
nmod=dim(ano_SLP_mod)[1] # models
nfc=dim(ano_SLP_mod)[2]  # members
nyr=dim(ano_SLP_mod)[3]  # startdates
nlat=dim(ano_SLP_mod)[5] # latitudes
nlon=dim(ano_SLP_mod)[6] # longitudes

# Seasonal means:
winter_ano_SLP_mod <- Mean1Dim(ano_SLP_mod[,,,fcavt,,],posdim=4)
winter_ano_SLP_obs <- Mean1Dim(ano_SLP_obs[,,,fcavt,,],posdim=2)

# Initialisation
NAOF.ver <- array(NA,c(nmod,nfc,nyr))
NAOO.ver <- array(NA,c(1,1,nyr))

# EOFs and PCs:
for (iy in 1:nyr) {
    # Observed EOF excluding the forecast start year (cross-validation).
    OEOF <- CFU_EOF(lon,lat,winter_ano_SLP_obs[-iy,,],neofs=1)
    # Choose a polarity of pattern similar to SLP pattern (positive over the Azores and negative over Iceland)
    sign <- 1; if ( 0 > mean(OEOF$EOFs[1,round(2*nlat/3),],na.rm=T) ) {sign <- -1}
    OEOF$EOFs <- OEOF$EOFs*sign; OEOF$PCs <- OEOF$PCs*sign
    # Plot first EOF for the first year:
    brks_def=seq(-5, 5, by=1)
    jBrewColors <- brewer.pal(n=length(brks_def)-1,name='RdBu')
    if (iy == 1) {
        postscript('first_EOF.ps')
        PlotEquiMap(OEOF$EOFs[1,,],lon,lat,filled.continents = FALSE,brks=brks_def,cols=rev(jBrewColors))
        dev.off()
    }
    if (naocalc == 1) { 
     for (imod in 1:nmod) {
       for (ifc in 1:nfc) {
         # This part has to be adapted according to the need of the user:
         # Here, the EOF is computed separetely for all models and members.
         # Sometime, it can be more correct to compute one single EOF per model for all members (more robust).
         # Anyway, it is also more common to use model projection onto observations.
         FEOF <- CFU_EOF(lon,lat,winter_ano_SLP_mod[imod,ifc,-iy,,],neofs=1)
         PlotEquiMap(FEOF$EOFs[1,,],lon,lat,filled.continents = FALSE,brks=brks_def,cols=rev(jBrewColors))
         # Polarity of pattern.
         sign <- 1; if ( 0 > mean(FEOF$EOFs[1,round(2*nlat/3),],na.rm=T) ) {sign <- -1}
         FEOF$EOFs <- FEOF$EOFs*sign; FEOF$PCs <- FEOF$PCs*sign
         PCF <- CFU_MODE(InsertDim(InsertDim(InsertDim(InsertDim(winter_ano_SLP_mod[imod,ifc,iy,,],1,1),1,1),1,1),4,1),FEOF,mode=1)
         NAOF.ver[imod,ifc,iy] <- PCF$PC.ver[1,1,1,1]
       }
     }
    } else if (naocalc == 0) {
     ## Project forecast anomalies on obs EOF
     PCF <- CFU_MODE(InsertDim(InsertDim(winter_ano_SLP_mod[,,iy,,],3,1),4,1),OEOF,mode=1)
     NAOF.ver[,,iy] <- PCF$PC.ver[,,1,1]
    } else {
     stop("Invalid value for naocalc, should be 0 or 1")
    }
    ## Project observed anomalies.
    PCO <- CFU_MODE(InsertDim(InsertDim(InsertDim(InsertDim(winter_ano_SLP_obs[iy,,],1,1),1,1),1,1),1,1),OEOF,mode=1)
    NAOO.ver[1,1,iy] <- PCO$PC.ver[1,1,1,1]
}

#######
# Plot:
#######

maxyr=17
postscript('NAO_EOF.ps', width = 550, height = 300)
ylimites=c(-600,600)
xlimites=c(0.5,maxyr+0.5)
plot(NULL,main='Winter NAO forecast in November - NAO defined as a projection on 1st EOF\n Models in colour, observation in black',xaxt = "n",xlab='Year',ylab='Projection on 1st EOF',xlim=xlimites,ylim=ylimites)
axis(1,at=1:maxyr,labels=seq(1993,2009,1))
abline(v=seq(0.5,maxyr+0.5,1),lty=3)
abline(h=1)

color1='red'
color2='orange'
color3='purple'
color4='green'

for (ji in 1:maxyr) {

# m04o
x=array(ji-0.3,dim=nfc)
# Plot index for members:
par(new=TRUE)
plot(x,NAOF.ver[1,,ji],cex=1,pch=1,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color1)
# Plot mean:
par(new=TRUE)
plot(ji-0.3,mean(NAOF.ver[1,,ji],na.rm='TRUE'),cex=3,pch=1,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color1)

# m04r
x=array(ji-0.15,dim=nfc)
par(new=TRUE)
# Plot index for members:
plot(x,NAOF.ver[2,,ji],cex=1,pch=5,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color2)
# Plot mean:
par(new=TRUE)
plot(ji-0.15,mean(NAOF.ver[2,,ji],na.rm='TRUE'),cex=3,pch=5,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color2)

# m04p
x=array(ji+0.15,dim=nfc)
par(new=TRUE)
# Plot index for members:
plot(x,NAOF.ver[3,,ji],cex=1,pch=2,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color3)
# Plot mean:
par(new=TRUE)
plot(ji+0.15,mean(NAOF.ver[3,,ji],na.rm='TRUE'),cex=3,pch=2,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color3)

# m04s
x=array(ji+0.3,dim=nfc)
par(new=TRUE)
# Plot index for members:
plot(x,NAOF.ver[4,,ji],cex=1,pch=7,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color4)
# Plot mean:
par(new=TRUE)
plot(ji+0.3,mean(NAOF.ver[4,,ji],na.rm='TRUE'),cex=3,pch=7,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color4)

# obs
par(new=TRUE)
plot(ji-0.3,NAOO.ver[1,1,ji],cex=3,pch=4,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col='black')

}

legend(12,650,legend=c('low resolution, soil moisture climatology','low resolution, observed soil moisture','high resolution, soil moisture climatology','high resolution, observed soil moisture', 'ERAinterim'),pch=c(1,5,2,7,4),col=c(color1,color2,color3,color4,'black'))
dev.off()

# Comparison with simple NAO definitions:
#########################################

print(c("limit between Iceland low pressure system and Azores anticyclone to compute NAO index: ",lat[nlat/2.]))

# Here we use the domain from Stephenson (2006):
NAO_mod <- Mean1Dim(Mean1Dim(winter_ano_SLP_mod[,,,(nlat/2+1):nlat,],5),4)-Mean1Dim(Mean1Dim(winter_ano_SLP_mod[,,,1:(nlat/2.),],5),4)
NAO_obs <- Mean1Dim(Mean1Dim(winter_ano_SLP_obs[,(nlat/2+1):nlat,],3),2)-Mean1Dim(Mean1Dim(winter_ano_SLP_obs[,1:(nlat/2.),],3),2)

#######
# Plot:
#######

maxyr=17
postscript('NAO_stephenson.ps', width = 550, height = 300)
ylimites=c(-1200,1200)
xlimites=c(0.5,maxyr+0.5)
plot(NULL,main='Winter NAO forecast in November, NAO defined on SLP difference 90°W:60°E (20°N:55°N - 55°N:90°N)\n Models in colour, observation in black',xaxt = "n",xlab='Year',ylab='Difference of SLP',xlim=xlimites,ylim=ylimites)
axis(1,at=1:maxyr,labels=seq(1993,2009,1))
abline(v=seq(0.5,maxyr+0.5,1),lty=3)
abline(h=1)

color1='red'
color2='orange'
color3='purple'
color4='green'

for (ji in 1:maxyr) {

# m04o
x=array(ji-0.3,dim=nfc)
# Plot index for members:
par(new=TRUE)
plot(x,NAO_mod[1,,ji],cex=1,pch=1,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color1)
# Plot mean:
par(new=TRUE)
plot(ji-0.3,mean(NAO_mod[1,,ji],na.rm='TRUE'),cex=3,pch=1,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color1)

# m04r
x=array(ji-0.15,dim=nfc)
par(new=TRUE)
plot(x,NAO_mod[2,,ji],cex=1,pch=5,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color2)
# Plot mean:
par(new=TRUE)
plot(ji-0.15,mean(NAO_mod[2,,ji],na.rm='TRUE'),cex=3,pch=5,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color2)

# m04p
x=array(ji+0.15,dim=nfc)
par(new=TRUE)
plot(x,NAO_mod[3,,ji],cex=1,pch=2,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color3)
# Plot mean:
par(new=TRUE)
plot(ji+0.15,mean(NAO_mod[3,,ji],na.rm='TRUE'),cex=3,pch=2,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color3)

# m04s
x=array(ji+0.3,dim=nfc)
par(new=TRUE)
plot(x,NAO_mod[4,,ji],cex=1,pch=7,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color4)
# Plot mean:
par(new=TRUE)
plot(ji+0.3,mean(NAO_mod[4,,ji],na.rm='TRUE'),cex=3,pch=7,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color4)

# obs
par(new=TRUE)
plot(ji-0.3,NAO_obs[ji],cex=3,pch=4,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col='black')

}

legend(12,1250,legend=c('low resolution, soil moisture climatology','low resolution, observed soil moisture','high resolution, soil moisture climatology','high resolution, observed soil moisture', 'ERAinterim'),pch=c(1,5,2,7,4),col=c(color1,color2,color3,color4,'black'))
dev.off()

# Comparison with simple NAO definition:
########################################

print(c("limit between Iceland low pressure system and Azores anticyclone to compute NAO index: ",lat[nlat/2.]))

# Here we use the smaller domain from Hurrel (2003), excluding latitudes from 80°N to 90°N:
NAO_mod <- Mean1Dim(Mean1Dim(winter_ano_SLP_mod[,,,(nlat/2+1):nlat,],5),4)-Mean1Dim(Mean1Dim(winter_ano_SLP_mod[,,,15:(nlat/2.),-(58:86)],5),4)
NAO_obs <- Mean1Dim(Mean1Dim(winter_ano_SLP_obs[,(nlat/2+1):nlat,],3),2)-Mean1Dim(Mean1Dim(winter_ano_SLP_obs[,15:(nlat/2.),-(58:86)],3),2)
#NAO_mod <- Mean1Dim(Mean1Dim(winter_ano_SLP_mod[,,,(nlat/2+1):nlat,],5),4)-Mean1Dim(Mean1Dim(winter_ano_SLP_mod[,,,1:(nlat/2.),],5),4)
#NAO_obs <- Mean1Dim(Mean1Dim(winter_ano_SLP_obs[,(nlat/2+1):nlat,],3),2)-Mean1Dim(Mean1Dim(winter_ano_SLP_obs[,1:(nlat/2.),],3),2)

#######
# Plot:
#######

maxyr=17
postscript('NAO_hurrel.ps', width = 550, height = 300)
ylimites=c(-1200,1200)
xlimites=c(0.5,maxyr+0.5)
plot(NULL,main='Winter NAO forecast in November, NAO defined on SLP difference 90°W:40°E (20°N:55°N - 55°N:80°N))\n Models in colour, observation in black',xaxt = "n",xlab='Year',ylab='Difference of SLP',xlim=xlimites,ylim=ylimites)
axis(1,at=1:maxyr,labels=seq(1993,2009,1))
abline(v=seq(0.5,maxyr+0.5,1),lty=3)
abline(h=1)

color1='red'
color2='orange'
color3='purple'
color4='green'

for (ji in 1:maxyr) {

# m04o
x=array(ji-0.3,dim=nfc)
# Plot index for members:
par(new=TRUE)
plot(x,NAO_mod[1,,ji],cex=1,pch=1,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color1)
# Plot mean:
par(new=TRUE)
plot(ji-0.3,mean(NAO_mod[1,,ji],na.rm='TRUE'),cex=3,pch=1,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color1)

# m04r
x=array(ji-0.15,dim=nfc)
#if (signif_EOF_1991[ji]) {color='red'} else {color='blue'}
par(new=TRUE)
plot(x,NAO_mod[2,,ji],cex=1,pch=5,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color2)
# Plot mean:
par(new=TRUE)
plot(ji-0.15,mean(NAO_mod[2,,ji],na.rm='TRUE'),cex=3,pch=5,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color2)

# m04p
x=array(ji+0.15,dim=nfc)
par(new=TRUE)
plot(x,NAO_mod[3,,ji],cex=1,pch=2,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color3)
# Plot mean:
par(new=TRUE)
plot(ji+0.15,mean(NAO_mod[3,,ji],na.rm='TRUE'),cex=3,pch=2,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color3)

# m04s
x=array(ji+0.3,dim=nfc)
#if (signif_EOF_2153[ji]) {color='red'} else {color='blue'}
par(new=TRUE)
plot(x,NAO_mod[4,,ji],cex=1,pch=7,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color4)
# Plot mean:
par(new=TRUE)
plot(ji+0.3,mean(NAO_mod[4,,ji],na.rm='TRUE'),cex=3,pch=7,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col=color4)

# obs
par(new=TRUE)
plot(ji-0.3,NAO_obs[ji],cex=3,pch=4,xlab="",ylab="",axes=FALSE,xlim=xlimites,ylim=ylimites,col='black')

}

legend(12,1250,legend=c('low resolution, soil moisture climatology','low resolution, observed soil moisture','high resolution, soil moisture climatology','high resolution, observed soil moisture', 'ERAinterim'),pch=c(1,5,2,7,4),col=c(color1,color2,color3,color4,'black'))
dev.off()

# Save data if needed (to redo the plot)
########################################
save(NAOO.ver,NAOF.ver,NAO_mod,NAO_obs,file = "NAO.RData")

# Correlation (not used here since it is never significant!
###########################################################

#cor_NAO <- Corr(InsertDim(Mean1Dim(NAOF.ver,2),2,1), NAOO.ver, posloop=1, poscor=3)
